import abc

from easytradesdk.entity.KlineInterval import KlineInterval
from easytradesdk.entity.Symbol import Symbol
from easytradesdk.entity.TC import TC


class AbsMarketApi(metaclass=abc.ABCMeta):

    def fetchKline(self, tc: TC, symbol: Symbol, interval: KlineInterval, startTimeMills=None, endTimeMills=None, limit=200):
        pass

    def fetchLatestKline(self, tc: TC, symbol: Symbol, interval: KlineInterval, endTimeMills=None, limit=200, excludeCurrent=None):
        pass

    def fetchDepth(self, tc: TC, symbol: Symbol, depthLimit=100):
        pass

    def fetchTicker(self, tc: TC, symbol: Symbol):
        pass



from enum import Enum


class Symbol(Enum):
    BTC_USDT = "BTC_USDT"
    ETH_BTC = "ETH_BTC"
    ETH_USDT = "ETH_USDT"

    @staticmethod
    def parse(symbol):
        result = list(filter(lambda c: c.value == symbol, Symbol))
        if result:
            return result[0]
        return None



from enum import Enum


class TC(Enum):
    BINANCE = "BINANCE"

    @staticmethod
    def parse(tc):
        result = list(filter(lambda c: c.value == tc, TC))
        if result:
            return result[0]
        return None



from enum import Enum


class KlineInterval(Enum):
    S_1m = "1m"
    S_3m = "3m"
    S_5m = "5m"
    S_15m = "15m"
    S_30m = "30m"
    S_1h = "1h"
    S_2h = "2h"
    S_4h = "4h"
    S_6h = "6h"
    S_8h = "8h"
    S_12h = "12h"
    S_1d = "1d"
    S_3d = "3d"
    S_1W = "1w"
    S_1M = "1M"

    @staticmethod
    def parse(interval):
        result = list(filter(lambda c: c.value == interval, KlineInterval))
        if result:
            return result[0]
        return None



import decimal

from easytradesdk.Serializer import DeserializableObject


class DepthValue(DeserializableObject):

    def __init__(self):
        self.price = None
        self.quantity = None

    def getObjectMapper(self):
        return {"price": decimal.Decimal, "quantity": decimal.Decimal}


class Depth(DeserializableObject):

    def __init__(self):
        self.bids = []
        self.asks = []

    def getObjectMapper(self):
        return {"bids": DepthValue, "asks": DepthValue}
